/* kluStor v.3.0.1
 *  Copyright (c) 1997 by Peter Johnson (locke@mcs.net).
 *  Read Doc/copying.pj for usage information.
 *
 * File: kluCmp.cpp
 * Desc: klu Compression interface
 */
#include <memory.h>
#include <zlib.h>

#include "kluCmp.h"

int DoCompCopy(void *, unsigned long *, const void *, unsigned long);
int DoCompzlib(void *, unsigned long *, const void *, unsigned long);
int DoCompBWT(void *, unsigned long *, const void *, unsigned long);

int DoDecompCopy(void *, unsigned long, const void *, unsigned long);
int DoDecompzlib(void *, unsigned long, const void *, unsigned long);
int DoDecompBWT(void *, unsigned long, const void *, unsigned long);

unsigned char NumKluCompress = 2;

kluCompressStruct kluCompress[] =
{
	{"None",	DoCompCopy,	DoDecompCopy},
	{"zlib",	DoCompzlib,	DoDecompzlib},
	{"BWT",		DoCompBWT,	DoDecompBWT }
};

int DoCompCopy(void *dest, unsigned long *destlen, const void *src,
	unsigned long srclen)
{
	memcpy(dest, src, srclen);
	*destlen = srclen;
	return 0;
}

int DoCompzlib(void *dest, unsigned long *destlen, const void *src,
	unsigned long srclen)
{
	if(compress((Bytef *)dest, destlen, (const Bytef *)src, srclen)!=Z_OK)
		return 1;
	return 0;
}

int DoCompBWT(void *dest, unsigned long *destlen, const void *src,
	unsigned long srclen)
{
	// BWT not currently implemented
}


int DoDecompCopy(void *dest, unsigned long destlen, const void *src,
	unsigned long srclen)
{
	memcpy(dest, src, destlen);
	return 0;
}

int DoDecompzlib(void *dest, unsigned long destlen, const void *src,
	unsigned long srclen)
{
	unsigned long temp = destlen;	// discard

	if(uncompress((Bytef *)dest, &temp, (const Bytef *)src, srclen)!=Z_OK)
		return 1;
	return 0;
}

int DoDecompBWT(void *dest, unsigned long destlen, const void *src,
	unsigned long srclen)
{
	// BWT not currently implemented
}
